@@ -1,17 +1,22 @@
 import re
+from sys import getsizeof
+
+from concurrent.futures.thread import ThreadPoolExecutor
+from asyncio import Future
 
 import boto3
 from boto3.dynamodb.conditions import Key, Attr
 from boto3.exceptions import ResourceNotExistsError
 from boto3.session import Session
-from typing import List, Optional, Type, Any, Dict
+from typing import List, Optional, Type, Any, Dict, Tuple
 
 from botocore.exceptions import ClientError
 from pydantic import BaseModel, validate_arguments
 from pydantic.dataclasses import dataclass
 
 from StructNoSQL.dynamodb.dynamodb_utils import Utils
-from StructNoSQL.dynamodb.models import DatabasePathElement, FieldSetter, DynamoDBMapObjectSetter
+from StructNoSQL.dynamodb.models import DatabasePathElement, FieldSetter, DynamoDBMapObjectSetter, MapItemInitializer, \
+    MapItemInitializerContainer
 from StructNoSQL.practical_logger import message_with_vars
 from StructNoSQL.safe_dict import SafeDict
 from StructNoSQL.utils.static_logger import logger
@@ -25,6 +30,7 @@ class GetItemResponse(BaseModel):
     item: Optional[dict]
     success: bool
 
+
 class Response:
     def __init__(self, response_dict: dict):
         response_safedict = SafeDict(response_dict)
@@ -35,6 +41,7 @@ class Response:
         self.last_evaluated_key = response_safedict.get("LastEvaluatedKey").to_dict(default=None)
         self.has_reached_end = False if self.last_evaluated_key is not None else True
 
+
 class Index(BaseModel):
     hash_key_name: str
     hash_key_variable_python_type: Type
@@ -46,6 +53,7 @@ class Index(BaseModel):
 class PrimaryIndex(Index):
     pass
 
+
 class GlobalSecondaryIndex(Index):
     # todo: add pydantic to this class
     PROJECTION_TYPE_USE_ALL = "ALL"
@@ -217,13 +225,15 @@ class DynamoDbCoreAdapter:
         """
         if self.create_table:
             create_table_query_kwargs = CreateTableQueryKwargs(table_name=self.table_name, billing_mode=self.billing_mode)
-
-            create_table_query_kwargs.add_hash_key(key_name=self.primary_index.hash_key_name,
-                                                   key_python_variable_type=self.primary_index.hash_key_variable_python_type)
-
+            create_table_query_kwargs.add_hash_key(
+                key_name=self.primary_index.hash_key_name,
+                key_python_variable_type=self.primary_index.hash_key_variable_python_type
+            )
             if self.primary_index.sort_key_name is not None and self.primary_index.sort_key_variable_python_type is not None:
-                create_table_query_kwargs.add_hash_key(key_name=self.primary_index.sort_key_name,
-                                                       key_python_variable_type=self.primary_index.sort_key_variable_python_type)
+                create_table_query_kwargs.add_hash_key(
+                    key_name=self.primary_index.sort_key_name,
+                    key_python_variable_type=self.primary_index.sort_key_variable_python_type
+                )
 
             if self.global_secondary_indexes is not None:
                 create_table_query_kwargs.add_all_global_secondary_indexes(global_secondary_indexes=self.global_secondary_indexes)
@@ -268,12 +278,12 @@ class DynamoDbCoreAdapter:
             table = self.dynamodb.Table(self.table_name)
             response = table.get_item(**kwargs)
             if "Item" in response:
-                return GetItemResponse(item=Utils.dynamodb_to_python_higher_level(dynamodb_object=response["Item"]), success=True)
+                processed_item = Utils.dynamodb_to_python_higher_level(dynamodb_object=response["Item"])
+                return GetItemResponse(item=processed_item, success=True)
             else:
                 return GetItemResponse(item=None, success=False)
         except ResourceNotExistsError:
-            raise Exception(f"DynamoDb table {self.table_name} do not exist or in the process"
-                            "of being created. Failed to get attributes from DynamoDb table.")
+            raise Exception(f"DynamoDb table {self.table_name} do not exist or in the process of being created. Failed to get attributes from DynamoDb table.")
         except Exception as e:
             print(f"Failed to retrieve attributes from DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
             return None
@@ -287,8 +297,7 @@ class DynamoDbCoreAdapter:
             response = table.update_item(**query_kwargs_dict)
             return Response(response)
         except ResourceNotExistsError:
-            raise Exception(f"DynamoDb table {self.table_name} do not exist or in the process"
-                            "of being created. Failed to get attributes from DynamoDb table.")
+            raise Exception(f"DynamoDb table {self.table_name} do not exist or in the process of being created. Failed to get attributes from DynamoDb table.")
         except ClientError as e:
             print(f"{e} - No element has been found for the update query : {query_kwargs_dict}")
             return None
@@ -296,7 +305,8 @@ class DynamoDbCoreAdapter:
             print(f"Failed to update attributes in DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
             return None
 
-    def add_data_elements_to_list(self, index_name: str, key_value: Any, object_path: str, element_values: List[dict]) -> Optional[Response]:
+    def add_data_elements_to_list(self, index_name: str, key_value: Any, object_path: str,
+                                  element_values: List[dict]) -> Optional[Response]:
         kwargs = {
             "TableName": self.table_name,
             "Key": {index_name: key_value},
@@ -330,22 +340,35 @@ class DynamoDbCoreAdapter:
             self, index_name: str, key_value: Any,
             targets_path_elements: List[List[DatabasePathElement]],
             retrieve_removed_elements: bool = False
-    ) -> Optional[Response]:
+    ) -> Optional[Dict[str, Any]]:
 
+        consumed_targets_path_elements: List[List[DatabasePathElement]] = list()
         expression_attribute_names_dict = dict()
         update_expression = "REMOVE "
 
-        num_targets = len(targets_path_elements)
         for i_target, target in enumerate(targets_path_elements):
             current_target_num_path_elements = len(target)
+            current_setter_attribute_names = dict()
+            current_remover_update_expression = ""
+
             for i_path_element, path_element in enumerate(target):
                 current_path_key = f"#target{i_target}_pathKey{i_path_element}"
-                update_expression += current_path_key
-                expression_attribute_names_dict[current_path_key] = path_element.element_key
+                current_remover_update_expression += current_path_key
+                current_setter_attribute_names[current_path_key] = path_element.element_key
                 if i_path_element + 1 < current_target_num_path_elements:
-                    update_expression += "."
-            if i_target + 1 < num_targets:
-                update_expression += ", "
+                    current_remover_update_expression += "."
+
+            complete_update_expression_bytes_size = getsizeof(update_expression)
+            current_setter_update_expression_bytes_size = getsizeof(current_remover_update_expression)
+            update_expression_bytes_size_if_setter_is_added = complete_update_expression_bytes_size + current_setter_update_expression_bytes_size
+            if update_expression_bytes_size_if_setter_is_added < EXPRESSION_MAX_BYTES_SIZE:
+                if i_target > 0:
+                    update_expression += ", "
+                expression_attribute_names_dict = {**expression_attribute_names_dict, **current_setter_attribute_names}
+                update_expression += current_remover_update_expression
+                consumed_targets_path_elements.append(target)
+            else:
+                break
 
         update_query_kwargs = {
             'TableName': self.table_name,
@@ -355,71 +378,123 @@ class DynamoDbCoreAdapter:
             'ExpressionAttributeNames': expression_attribute_names_dict,
         }
         response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
-        if response is not None and response.attributes is not None:
-            response.attributes = Utils.dynamodb_to_python_higher_level(response.attributes)
-        return response
+        if response is None:
+            return None
 
-    def initialize_all_elements_in_map_target(
-            self, index_name: str, key_value: Any, field_path_elements: List[DatabasePathElement],
-            last_item_custom_overriding_init_value: Optional[Any] = None
+        # Even if we return an empty output_response_attributes dict, we do not want to return None instead of a dict, because since this function only returns a dict, a return
+        # value of None indicates that the operation failed. Where as, for example in delete operation, we will not request the removed attributes from the database, which will
+        # give us an empty output_response_attributes dict, but the delete operation will base itself on the presence of the dict to judge if the operation failed or not.
+        output_response_attributes: Dict[str, Any] = Utils.dynamodb_to_python_higher_level(response.attributes) if response.attributes is not None else dict()
+        if len(targets_path_elements) == len(consumed_targets_path_elements):
+            return output_response_attributes
+        else:
+            for current_target_path_elements in consumed_targets_path_elements:
+                targets_path_elements.remove(current_target_path_elements)
+
+            request_continuation_response_attributes = self.remove_data_elements_from_map(
+                index_name=index_name, key_value=key_value,
+                targets_path_elements=targets_path_elements,
+                retrieve_removed_elements=retrieve_removed_elements
+            )
+            combined_output_response_attributes = {
+                **(output_response_attributes or dict()),
+                **(request_continuation_response_attributes or dict())
+            }
+            return combined_output_response_attributes
+
+    @staticmethod
+    def _add_database_path_element_to_string_expression(
+            base_string: Optional[str], database_path_element: DatabasePathElement, path_key: str
+    ) -> Tuple[str, Dict[str, str]]:
+
+        output_string: str = base_string or str()
+        output_expression_attribute_names: Dict[str, str] = dict()
+
+        if isinstance(database_path_element.element_key, str):
+            if len(output_string) > 0:
+                output_string += "."
+            output_string += path_key
+            output_expression_attribute_names[path_key] = database_path_element.element_key
+        elif isinstance(database_path_element.element_key, int):
+            # If the element_key is an int, it means we try to access an index of a list or set. We can right away use the index access
+            # quotation ( [$index] instead of .$attributeName ) and not need to pass the index as an attribute name. Note, that anyway,
+            # boto3 will only accept strings as attribute names, and trying to pass an int or float as attribute name, will crash the request.
+            output_string += f"[{database_path_element.element_key}]"
+
+        return output_string, output_expression_attribute_names
+
+    @staticmethod
+    def _add_database_path_element_to_string_path(base_string: Optional[str], database_path_element: DatabasePathElement) -> str:
+        output_string: str = base_string or str()
+        if isinstance(database_path_element.element_key, str):
+            if len(output_string) > 0:
+                output_string += "."
+            output_string += database_path_element.element_key
+        elif isinstance(database_path_element.element_key, int):
+            # See comment on similar block code in function _add_database_path_element_to_string_expression
+            output_string += f"[{database_path_element.element_key}]"
+        return output_string
+
+    @staticmethod
+    def _prepare_map_initialization(
+            i: int, current_path_element: DatabasePathElement, overriding_init_value: Any,
+            previous_element_initializer_container: Optional[MapItemInitializerContainer] = None
+    ) -> MapItemInitializer:
+
+        current_path_key = f"#pathKey{i}"
+        current_path_target = str() if previous_element_initializer_container is None else previous_element_initializer_container.item.path_target
+
+        current_path_target, new_expression_attribute_names = DynamoDbCoreAdapter._add_database_path_element_to_string_expression(
+            base_string=current_path_target, database_path_element=current_path_element, path_key=current_path_key
+        )
+        expression_attribute_names = (
+            {**previous_element_initializer_container.item.expression_attribute_names, **new_expression_attribute_names}
+            if previous_element_initializer_container is not None else new_expression_attribute_names
+        )
+
+        item_default_value = overriding_init_value or current_path_element.get_default_value()
+        return MapItemInitializer(
+            path_target=current_path_target, last_item_element_key=current_path_element.element_key,
+            item_default_value=item_default_value, expression_attribute_names=expression_attribute_names
+        )
+
+    def initialize_element_in_map_target(
+            self, index_name: str, key_value: Any, initializer: MapItemInitializer, is_last_path_element: bool = False
     ) -> Optional[Response]:
 
-        current_path_target = ""
-        current_path_element: Optional[DatabasePathElement] = None
-        current_item_default_value: Any = None
-        expression_attribute_names_dict: Dict[str, str] = dict()
-        last_response: Optional[Response] = None
-
-        for i, current_path_element in enumerate(field_path_elements):
-            is_last_path_element = i >= (len(field_path_elements) - 1)
-            if i > 0:
-                current_path_target += "."
-
-            if i + 1 >= len(field_path_elements) and last_item_custom_overriding_init_value is not None:
-                current_item_default_value = last_item_custom_overriding_init_value
-            else:
-                current_item_default_value = current_path_element.get_default_value()
-
-            current_path_key = f"#pathKey{i}"
-            current_path_target += current_path_key
-            expression_attribute_names_dict[current_path_key] = current_path_element.element_key
-
-            current_update_expression = f"SET {current_path_target} = if_not_exists({current_path_target}, :item)"
-            current_set_potentially_missing_object_query_kwargs = {
-                "TableName": self.table_name,
-                "Key": {index_name: key_value},
-                "ReturnValues": "UPDATED_NEW" if is_last_path_element else "NONE",
-                # We do not need to waste retrieving the UPDATED_NEW value when we know we we
-                # will it only for the last path element (we use right after the loop ended)
-                "UpdateExpression": current_update_expression,
-                "ExpressionAttributeNames": expression_attribute_names_dict,
-                "ExpressionAttributeValues": {
-                    ":item": current_item_default_value
-                }
-            }
-            last_response = self._execute_update_query(query_kwargs_dict=current_set_potentially_missing_object_query_kwargs)
-
-        if current_path_element is not None:
+        base_query_kwargs = {
+            'TableName': self.table_name,
+            'Key': {index_name: key_value},
+            'ExpressionAttributeNames': initializer.expression_attribute_names,
+            'ExpressionAttributeValues': {
+                ':item': initializer.item_default_value
+            },
+            'ReturnValues': 'UPDATED_NEW' if is_last_path_element else 'NONE'
+            # We do not need to waste retrieving the UPDATED_NEW value when we know we we
+            # will it only for the last path element (we use right after the loop ended)
+        }
+        current_update_expression = f"SET {initializer.path_target} = if_not_exists({initializer.path_target}, :item)"
+        current_set_potentially_missing_object_query_kwargs = {**base_query_kwargs, 'UpdateExpression': current_update_expression}
+        response = self._execute_update_query(query_kwargs_dict=current_set_potentially_missing_object_query_kwargs)
+        if is_last_path_element is True:
             # If the last item attribute value in the database does (retrieved thanks to the "UPDATED_NEW" ReturnValues),
             # do not equal its default_value (which is either the field type default value, or the
             # last_item_custom_overriding_init_value parameter), we want to initialize again the item, because this means
             # that the attribute existed, so the SET update expression with if_not_exists did not performed, and we do
             # not currently have the value we want in the database. We perform this operation only for the last path
             # element in the request, to avoid overriding and scratching list or dictionary that are expected to exist.
-            field_existing_attribute_in_database = last_response.attributes.get(current_path_element.element_key)
-            if field_existing_attribute_in_database != current_item_default_value:
+            field_existing_attribute_in_database = response.attributes.get(initializer.last_item_element_key)
+            if field_existing_attribute_in_database != initializer.item_default_value:
                 # It is possible for the field to already exist, and yet, to not have
-                initialization_response = self._set_update_data_element_to_map_without_default_initialization(
-                    index_name=index_name, key_value=key_value,
-                    field_path_elements=field_path_elements,
-                    value=last_item_custom_overriding_init_value
-                )
-                return initialization_response
-        return last_response
+                current_update_expression = f"SET {initializer.path_target} = :item"
+                current_override_existing_object_value_query_kwargs = {**base_query_kwargs, 'UpdateExpression': current_update_expression}
+                override_object_value_response = self._execute_update_query(query_kwargs_dict=current_override_existing_object_value_query_kwargs)
+                return override_object_value_response
+        return response
 
-    def _set_update_data_element_to_map_without_default_initialization(
+    def _construct_update_data_element_to_map_query_kwargs(
             self, index_name: str, key_value: Any, field_path_elements: List[DatabasePathElement], value: Any
-    ) -> Optional[Response]:
+    ) -> dict:
 
         expression_attribute_names_dict = dict()
         update_expression = "SET "
@@ -443,51 +518,124 @@ class DynamoDbCoreAdapter:
                 ":item": value
             }
         }
+        return update_query_kwargs
+
+    def set_update_data_element_to_map_with_default_initialization(
+            self, index_name: str, key_value: Any, field_path_elements: List[DatabasePathElement], value: Any
+    ) -> Optional[Response]:
+        update_query_kwargs = self._construct_update_data_element_to_map_query_kwargs(
+            index_name=index_name, key_value=key_value, field_path_elements=field_path_elements, value=value
+        )
+        return self._execute_update_query_with_initialization_if_missing(
+            index_name=index_name, key_value=key_value, update_query_kwargs=update_query_kwargs,
+            setters=[DynamoDBMapObjectSetter(field_path_elements=field_path_elements, value_to_set=value)]
+        )
+
+    def set_update_data_element_to_map_without_default_initialization(
+            self, index_name: str, key_value: Any, field_path_elements: List[DatabasePathElement], value: Any
+    ) -> Optional[Response]:
+        update_query_kwargs = self._construct_update_data_element_to_map_query_kwargs(
+            index_name=index_name, key_value=key_value, field_path_elements=field_path_elements, value=value
+        )
         return self._execute_update_query(query_kwargs_dict=update_query_kwargs)
 
-    def set_update_data_element_to_map(
-            self, index_name: str, key_value: Any, value: Any, field_path_elements: List[DatabasePathElement]
+    @staticmethod
+    def _setters_to_tidied_initializers(setters: List[DynamoDBMapObjectSetter]) -> Dict[str, MapItemInitializerContainer]:
+        root_initializers_containers: Dict[str, MapItemInitializerContainer] = dict()
+        all_initializers_containers: Dict[str, MapItemInitializerContainer] = dict()
+
+        for setter in setters:
+            current_absolute_target_path = str()
+            last_map_initializer_container: Optional[MapItemInitializerContainer] = None
+
+            for i_path, path_element in enumerate(setter.field_path_elements):
+                current_absolute_target_path = DynamoDbCoreAdapter._add_database_path_element_to_string_path(
+                    base_string=current_absolute_target_path, database_path_element=path_element
+                )
+                existing_container: Optional[MapItemInitializerContainer] = all_initializers_containers.get(current_absolute_target_path, None)
+                if existing_container is None:
+                    overriding_init_value: Optional[Any] = setter.value_to_set if i_path + 1 >= len(setter.field_path_elements) else None
+                    current_map_initializer = DynamoDbCoreAdapter._prepare_map_initialization(
+                        i=i_path, current_path_element=path_element, overriding_init_value=overriding_init_value,
+                        previous_element_initializer_container=last_map_initializer_container
+                    )
+                    current_map_initializer_container = MapItemInitializerContainer(item=current_map_initializer, nexts_in_line=dict())
+                    all_initializers_containers[path_element.element_key] = current_map_initializer_container
+
+                    if last_map_initializer_container is None:
+                        # If the last_map_initializer_container is None, this means that we are in the first iteration of the
+                        # field_path_elements loop, which means that the current_map_initializer_container is a root initializer.
+                        root_initializers_containers[path_element.element_key] = current_map_initializer_container
+                    else:
+                        # If the last_map_initializer_container is not None, we know that our current_map_initializer_container is not
+                        # a root initializer, and needs to be set in the nexts_in_line variable of the last_map_initializer_container.
+                        last_map_initializer_container.nexts_in_line[path_element.element_key] = current_map_initializer_container
+                    last_map_initializer_container = current_map_initializer_container
+                else:
+                    last_map_initializer_container = existing_container
+                    logger.info(message_with_vars(
+                        message="Successfully tidied duplicate DatabasePathElement initializer",
+                        vars_dict={
+                            'trimmedPathElementKey': path_element.element_key,
+                            'trimmedSetterValueToSet': setter.value_to_set,
+                            'existingContainerItemPathTarget': existing_container.item.path_target,
+                            'existingContainerItemDefaultValue': existing_container.item.item_default_value
+                        }
+                    ))
+        return root_initializers_containers
+
+    def _initializer_task_executor(self, index_name: str, key_value: str, initializer_container: MapItemInitializerContainer) -> Optional[Response]:
+        # todo: group fields initialization in single request
+        is_last_path_element = not len(initializer_container.nexts_in_line) > 0
+        initialization_response = self.initialize_element_in_map_target(
+            index_name=index_name, key_value=key_value,
+            initializer=initializer_container.item,
+            is_last_path_element=is_last_path_element
+        )
+        if initialization_response is None:
+            logger.error(message_with_vars(
+                message="Initialized a field after a set/update multiple data elements in map request had failed.",
+                vars_dict={'initializer_container': initializer_container}
+            ))
+
+        if len(initializer_container.nexts_in_line) > 0:
+            results = self._run_initializers_in_executors(
+                index_name=index_name, key_value=key_value,
+                initializers=initializer_container.nexts_in_line
+            )
+        return initialization_response
+
+    def _run_initializers_in_executors(self, index_name: str, key_value: str, initializers: Dict[str, MapItemInitializerContainer]) -> Optional[List[Optional[Response]]]:
+        initializers_values = initializers.values()
+        num_initializers = len(initializers_values)
+        with ThreadPoolExecutor(max_workers=num_initializers) as executor:
+            results: List[Optional[Response]] = [executor.submit(
+                self._initializer_task_executor, index_name, key_value, item
+            ).result() for i, item in enumerate(initializers_values)]
+        return results
+
+    def _execute_update_query_with_initialization_if_missing(
+            self, index_name: str, key_value: Any, update_query_kwargs: dict, setters: List[DynamoDBMapObjectSetter]
     ) -> Optional[Response]:
 
-        response = self._set_update_data_element_to_map_without_default_initialization(
-            index_name=index_name, key_value=key_value, value=value, field_path_elements=field_path_elements
-        )
-        if response is None:
-            # If the response is None, it means that one of the path of the target path has not been found and need to be initialized.
-            print(message_with_vars(
-                message="Set/update request failed. Trying to initialize and populate the required objects.",
-                vars_dict={'index_name': index_name, 'key_value': key_value, 'field_path_elements': field_path_elements}
-            ))
-            return self.initialize_all_elements_in_map_target(
-                index_name=index_name, key_value=key_value,
-                field_path_elements=field_path_elements,
-                last_item_custom_overriding_init_value=value
-            )
-        return response
-
-    def _execute_update_query_with_initialization_if_missing(self, index_name: str, key_value: Any, update_query_kwargs: dict,
-                                                             setters: List[DynamoDBMapObjectSetter]) -> Optional[Response]:
         response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
         if response is None:
-            # If the response is None, it means that one of the path of the
-            # target path has not been found and need to be initialized.
-            last_response: Optional[Response] = None
-            for i_setter, current_setter in enumerate(setters):
-                # todo: group fields initialization in single request
-                last_response = self.initialize_all_elements_in_map_target(
-                    index_name=index_name, key_value=key_value,
-                    field_path_elements=current_setter.field_path_elements,
-                    last_item_custom_overriding_init_value=current_setter.value_to_set
-                )
-                if last_response is None:
-                    print(message_with_vars(
-                        message="Initialized a field after a set/update multiple data elements in map request had failed.",
-                        vars_dict={"fieldTargetPathElements": current_setter.field_path_elements}
-                    ))
+            # If the response is None, it means that one of the path of the target path has not been found and need to be initialized.
+            database_paths_initializers = self._setters_to_tidied_initializers(setters=setters)
+            results = self._run_initializers_in_executors(
+                index_name=index_name, key_value=key_value,
+                initializers=database_paths_initializers
+            )
+            last_response = results[-1]
+            # This function can only be triggered if there is at least one item in setters List, so we do not need to worry about accessing
+            # the last element in result List without checking if the len of the setters or of the results is superior to zero.
             return last_response
         return response
 
-    def set_update_multiple_data_elements_to_map(self, index_name: str, key_value: Any, setters: List[DynamoDBMapObjectSetter]) -> Optional[Response]:
+    def set_update_multiple_data_elements_to_map(
+            self, index_name: str, key_value: Any, setters: List[DynamoDBMapObjectSetter]
+    ) -> Optional[Response]:
+
         if not len(setters) > 0:
             # If we tried to run the query with no object setter,
             # she will crash when executed. So we return None.
@@ -499,22 +647,20 @@ class DynamoDbCoreAdapter:
             "ReturnValues": "UPDATED_NEW"
         }
         update_expression = "SET "
-        expression_attribute_names_dict = dict()
-        expression_attribute_values_dict = dict()
+        expression_attribute_names_dict, expression_attribute_values_dict = dict(), dict()
 
-        from sys import getsizeof
         consumed_setters: List[DynamoDBMapObjectSetter] = list()
         for i_setter, current_setter in enumerate(setters):
             current_setter_update_expression = ""
-            current_setter_attribute_names = dict()
-            current_setter_attribute_values = dict()
+            current_setter_attribute_names, current_setter_attribute_values = dict(), dict()
             for i_path, current_path_element in enumerate(current_setter.field_path_elements):
                 current_path_key = f"#setter{i_setter}_pathKey{i_path}"
-                current_setter_update_expression += current_path_key
-                current_setter_attribute_names[current_path_key] = current_path_element.element_key
-                if i_path + 1 < len(current_setter.field_path_elements):
-                    current_setter_update_expression += "."
-                else:
+                current_setter_update_expression, new_expression_attribute_names = DynamoDbCoreAdapter._add_database_path_element_to_string_expression(
+                    base_string=current_setter_update_expression, database_path_element=current_path_element, path_key=current_path_key
+                )
+                current_setter_attribute_names = {**current_setter_attribute_names, **new_expression_attribute_names}
+
+                if i_path >= (len(current_setter.field_path_elements) - 1):
                     current_setter_update_expression += f" = :item{i_setter}"
                     current_setter_attribute_values[f":item{i_setter}"] = current_setter.value_to_set
 
@@ -565,7 +711,8 @@ class DynamoDbCoreAdapter:
             return self.set_update_multiple_data_elements_to_map(index_name=index_name, key_value=key_value, setters=setters)
 
     def query_by_key(
-            self, index_name: str, key_value: Any, fields_paths_elements: Optional[List[List[DatabasePathElement]]] = None,
+            self, index_name: str, key_value: Any,
+            fields_paths_elements: Optional[List[List[DatabasePathElement]]] = None,
             filter_expression: Optional[Any] = None, query_limit: Optional[int] = None, **additional_kwargs
     ) -> Response:
         if fields_paths_elements is not None:
@@ -596,8 +743,11 @@ class DynamoDbCoreAdapter:
             raise Exception(f"Failed to retrieve attributes from DynamoDb table."
                             f"Exception of type {type(e).__name__} occurred: {str(e)}")
 
-    def query_single_item_by_key(self, index_name: str, key_value: Any, fields_paths_elements: Optional[List[List[DatabasePathElement]]] = None,
-                                 filter_expression: Optional[Any] = None) -> Optional[dict]:
+    def query_single_item_by_key(
+            self, index_name: str, key_value: Any,
+            fields_paths_elements: Optional[List[List[DatabasePathElement]]] = None,
+            filter_expression: Optional[Any] = None
+    ) -> Optional[dict]:
         # Yes, a query request is heavier than a get request that we could do with the _get_item_by_primary_key function.
         # Yet, in a get request, we cannot specify an index_name to query on. So, the _query_single_item_by_key should be
         # used when we want to get an item based on another index that the primary one. Otherwise, use _get_item_by_primary_key
@@ -643,32 +793,6 @@ class DynamoDbCoreAdapter:
                 else:
                     return None
 
-    def navigate_into_data_with_field_path_elements(
-            self, data: Any, field_path_elements: List[DatabasePathElement], num_keys_to_navigation_into: int
-    ) -> Optional[Any]:
-        if data is not None:
-            num_field_path_elements = len(field_path_elements)
-            for i, path_element in enumerate(field_path_elements):
-                if i + 1 > num_keys_to_navigation_into:
-                    break
-                elif i > 0 and field_path_elements[i - 1].default_type == list:
-                    data = data[0]
-                else:
-                    if not isinstance(data, dict):
-                        break
-
-                    if path_element.default_type == set and data == {} and num_field_path_elements > i + 1:
-                        data = field_path_elements[i + 1].element_key
-                        break
-
-                    data = data.get(path_element.element_key, None)
-                    if data is None:
-                        # If the response_item is None, it means that one key has not been found,
-                        # so we need to break the loop in order to try to call get on a None object,
-                        # and then we will return the response_item, so we will return None.
-                        break
-        return data
-
     def get_data_in_path_target(
             self, index_name: str, key_value: str,
             field_path_elements: List[DatabasePathElement],
@@ -700,7 +824,8 @@ class DynamoDbCoreAdapter:
 
     def get_data_from_multiple_fields_in_path_target(
             self, key_value: str, fields_paths_elements: Dict[str, List[DatabasePathElement]],
-            num_keys_to_stop_at_before_reaching_end_of_item: int, index_name: Optional[str] = None, metadata: bool = False
+            num_keys_to_stop_at_before_reaching_end_of_item: int, index_name: Optional[str] = None,
+            metadata: bool = False
     ) -> Optional[Dict[str, Any]]:
 
         response_item = self.get_or_query_single_item(
@@ -816,3 +941,28 @@ class DynamoDbCoreAdapter:
             output_kwargs["ProjectionExpression"] = projection_expression
 
         return output_kwargs
+
+    @staticmethod
+    def navigate_into_data_with_field_path_elements(data: Any, field_path_elements: List[DatabasePathElement], num_keys_to_navigation_into: int) -> Optional[Any]:
+        if data is not None:
+            num_field_path_elements = len(field_path_elements)
+            for i, path_element in enumerate(field_path_elements):
+                if i + 1 > num_keys_to_navigation_into:
+                    break
+                elif i > 0 and field_path_elements[i - 1].default_type == list:
+                    data = data[0]
+                else:
+                    if not isinstance(data, dict):
+                        break
+
+                    if path_element.default_type == set and data == {} and num_field_path_elements > i + 1:
+                        data = field_path_elements[i + 1].element_key
+                        break
+
+                    data = data.get(path_element.element_key, None)
+                    if data is None:
+                        # If the response_item is None, it means that one key has not been found,
+                        # so we need to break the loop in order to try to call get on a None object,
+                        # and then we will return the response_item, so we will return None.
+                        break
+        return data
